@@ -1,32 +1,35 @@
 """ Module containing BigQueryExecutor for Python """
-import os
-import re
-import logging
-import time
 import json
+import logging
+import os
 import pandas as pd
-from pandas._testing import assert_frame_equal
+import re
+import time
+from google.api_core import exceptions
 from google.cloud import bigquery
 from google.cloud.exceptions import NotFound
 from google.oauth2 import service_account
-from google.api_core import exceptions
-from pygyver.etl.lib import bq_token_file_valid
-from pygyver.etl.lib import bq_token_file_path
-from pygyver.etl.lib import bq_default_project
-from pygyver.etl.lib import bq_default_dataset
-from pygyver.etl.lib import gcs_default_bucket
-from pygyver.etl.lib import read_table_schema_from_file
-from pygyver.etl.lib import bq_start_date
-from pygyver.etl.lib import bq_end_date
-from pygyver.etl.lib import set_write_disposition, set_priority
+from pandas._testing import assert_frame_equal
+from pygyver.etl.db import DBExecutor
+from pygyver.etl.lib import (
+    bq_default_dataset,
+    bq_default_location,
+    bq_default_project,
+    bq_end_date,
+    bq_start_date,
+    bq_token_file_path,
+    bq_token_file_valid,
+    gcs_default_bucket,
+    read_table_schema_from_file,
+    set_priority,
+    set_write_disposition
+)
+from pygyver.etl.storage import GCSExecutor
 from pygyver.etl.toolkit import (
     date_lister,
     flatten_df_columns,
     validate_date
 )
-from pygyver.etl.gs import load_gs_to_dataframe
-from pygyver.etl.storage import GCSExecutor
-from pygyver.etl.db import DBExecutor
 
 
 class BigQueryExecutorError(Exception):
@@ -207,16 +210,21 @@ class BigQueryExecutor:
         except exceptions.Conflict as error:
             logging.error(error)
 
-
-    def create_dataset(self, dataset_id=bq_default_dataset(), project_id=bq_default_project(), **kwargs):
-        """ Creates a BigQuery dataset if the dataset does not exists. Otherwise pass.
+    def create_dataset(
+            self,
+            dataset_id=bq_default_dataset(),
+            project_id=bq_default_project(),
+            location=bq_default_location()
+    ):
+        """Creates a BigQuery dataset if the dataset does not exists. Otherwise pass.
 
         Parameters:
             dataset_id (string): BigQuery dataset ID.
             project_id (string): BigQuery project ID.
+            location (string): BigQuery location - if empty takes the project's default location
         """
-        dataset_ref = self.get_dataset_ref(dataset_id,project_id=project_id)
-        if self.dataset_exists(dataset_id,project_id=project_id):
+        dataset_ref = self.get_dataset_ref(dataset_id, project_id=project_id)
+        if self.dataset_exists(dataset_id, project_id=project_id):
             logging.info(
                 "Dataset %s already exists in project %s",
                 dataset_id,
@@ -224,11 +232,15 @@ class BigQueryExecutor:
             )
         else:
             try:
-                self.client.create_dataset(dataset_ref)
+                dataset = bigquery.Dataset(dataset_ref)
+                dataset.location = location
+                self.client.create_dataset(dataset)
+                location_name = location if location else "default"
                 logging.info(
-                    "Created dataset %s in in project %s",
+                    "Created dataset %s in project %s and %s location",
                     dataset_id,
-                    project_id
+                    project_id,
+                    location_name
                 )
             except exceptions.Conflict as error:
                 logging.error(error)
@@ -336,7 +348,7 @@ class BigQueryExecutor:
 
     def create_table(self, table_id, dataset_id=bq_default_dataset(),project_id=bq_default_project(), sql=None, file=None,
                      write_disposition='WRITE_TRUNCATE', use_legacy_sql=False,
-                     location='US', schema_path='',
+                     location=bq_default_location(), schema_path='',
                      partition=False,
                      partition_field='_PARTITIONTIME',
                      clustering=None,
@@ -345,7 +357,7 @@ class BigQueryExecutor:
                      increment_field=None,
                      **kwargs):
         """ create a bigquery table from a sql query or file
-        
+
         Parameters:
             table_id:
             dataset_id:
@@ -405,7 +417,7 @@ class BigQueryExecutor:
         job_config.create_disposition = bigquery.CreateDisposition.CREATE_IF_NEEDED
         job_config.priority = set_priority(priority)
         job_config.clustering_fields = clustering
-        
+
         if partition:
             if not partition_field or partition_field == '_PARTITIONTIME':
                 job_config.time_partitioning = bigquery.table.TimePartitioning(
@@ -417,7 +429,7 @@ class BigQueryExecutor:
                 )
             else:
                 raise ValueError("partition_field should be a string")
-        
+
         query_job = self.client.query(
             sql,
             location=location,
@@ -437,11 +449,11 @@ class BigQueryExecutor:
 
         return query_job
 
-    def check_increment_dates(self, 
-                              dataset_id, 
-                              table_id, 
-                              increment_field, 
-                              from_date, 
+    def check_increment_dates(self,
+                              dataset_id,
+                              table_id,
+                              increment_field,
+                              from_date,
                               to_date):
         """ Checks whether increment dates to be added are in existing table.
 
@@ -459,7 +471,7 @@ class BigQueryExecutor:
                   FROM
                     `{dataset_id}.{table_id}`
                   WHERE
-                    {increment_field} BETWEEN 
+                    {increment_field} BETWEEN
                     CAST('{from_date}' AS DATE) AND CAST('{to_date}' AS DATE)
                 """
         result = self.execute_sql(
@@ -709,7 +721,9 @@ class BigQueryExecutor:
         """
         logging.info("Starting backfill")
 
-        backfill_dataset = "backfill"
+        # handling the different bq locations
+        bq_loc = bq_default_location().lower()
+        backfill_dataset = "backfill" if not bq_loc else f"backfill_{bq_loc}"
 
         if backfill not in ("full", "partial"):
             raise BigQueryExecutorError("BACKFILL env var needs to be 'full' or 'partial'")
@@ -826,7 +840,7 @@ class BigQueryExecutor:
 
         New:
         pipeline_partition_date is required for a normal pipeline run and is a single date.
-        To run a backfill, the BACKFILL env var needs to be set to 'full' or 'partial' and an optional date range 
+        To run a backfill, the BACKFILL env var needs to be set to 'full' or 'partial' and an optional date range
         can be set for a partial backfill by setting the FROM_PARTITION and TO_PARTITION env vars.
         """
         if sql is None and file is None:
@@ -875,7 +889,7 @@ class BigQueryExecutor:
                     **kwargs
                 )
 
-                # change date format from YYYY-MM-DD to YYYYMMDD for partition_id 
+                # change date format from YYYY-MM-DD to YYYYMMDD for partition_id
                 partition_id = pipeline_partition_date.replace("-", "")
                 partition_name = self.set_partition_name(
                     table=table_id,
@@ -1277,18 +1291,18 @@ class BigQueryExecutor:
 
         return data
 
-    def execute_dml(self, sql=None, file=None, location='US', project=bq_default_project()):
+    def execute_dml(self, sql=None, file=None, location=bq_default_location(), project=bq_default_project()):
 
         job_config = bigquery.QueryJobConfig()
 
         try:
 
             sql = sql if sql else read_sql(file)
-        
+
             job = self.client.query(sql, location=location, job_config=job_config)
-            
+
             results = job.result()
-            
+
             logging.info("%s - %s - %s rows effected (%s elapsed, %s B processed)",
                 job.statement_type,
                 job.state,
@@ -1296,7 +1310,7 @@ class BigQueryExecutor:
                 job.ended - job.started,
                 job.total_bytes_processed
             )
-        
+
         except Exception as e:
             logging.error(e)
             raise e
@@ -1516,7 +1530,7 @@ class BigQueryExecutor:
                 job = self.client.load_table_from_file(
                     file_obj=data,
                     destination=table_ref,
-                    location='US',
+                    location=bq_default_location(),
                     job_config=job_config
                 )
                 job.result()
@@ -1556,7 +1570,7 @@ class BigQueryExecutor:
             job = self.client.load_table_from_json(
                 json_rows=json,
                 destination=table_ref,
-                location='US',
+                location=bq_default_location(),
                 job_config=job_config
             )
             job.result()
