@@ -94,6 +94,13 @@ def bq_default_project():
     return os.environ.get('BIGQUERY_PROJECT', '')
 
 
+def bq_default_location():
+    """
+    Returns BIGQUERY_LOCATION if env is set, otherwise defaults to US
+    """
+    return os.environ.get('BIGQUERY_LOCATION', 'US')
+
+
 def bq_default_prod_project():
     """
     Returns BIGQUERY_PROD_PROJECT if env is set or 'copper-actor-127213' if not
