@@ -429,7 +429,9 @@ def get_env_var(
         raise ValueError(f"Env var '{env_var_name}' doesn't exist")
 
     if allowed_values:
-        assert env_var_name in allowed_values, f"Env var '{env_var_name}' is\
-            not in allowed values list '{allowed_values}'"
+        assert env_var in allowed_values, (
+            f"Env var value for '{env_var_name}' is not in allowed values "
+            f"list '{allowed_values}'"
+        )
 
     return env_var
